import snakypy.file
import snakypy.path
import snakypy.json
import snakypy.utils
import snakypy.console
import snakypy.ansi
import snakypy.calc
from snakypy.utils import catch
from snakypy.utils import stiff
from snakypy.utils import decorators
from snakypy.ansi import FG, BG, SGR
from snakypy.console import printer, entry, pick


__author__ = {
    'name': 'Snakypy Org',
    'email': 'contact.snakypy@gmail.com',
    'website': 'https://snakypy.github.io',
    'github': 'https://github.com/snakypy'
}
__version__ = '0.2.1'
